from config import *
import pandas as pd
import pickle
import xlearn as xl
import datetime
from userProfile import fetch_user_profile


# 接收device_id、city_id


# 将device_id、city_id拼接到对应的城市热门日记表。注意：下面预测集特征顺序要与训练集保持一致
def feature_en(user_profile):
    file_name = DIRECTORY_PATH + "diaryTestSet/{0}DiaryTop2000.csv".format(user_profile['city_id'])
    data = pd.read_csv(file_name)

    data["device_id"] = user_profile['device_id']

    now = datetime.datetime.now()
    data["hour"] = now.hour
    data["minute"] = now.minute

    data.loc[data["hour"] == 0, ["hour"]] = 24
    data.loc[data["minute"] == 0, ["minute"]] = 60
    data["hour"] = data["hour"].astype("category")
    data["minute"] = data["minute"].astype("category")

    data["y"] = 0
    data = data.drop("city_id", axis=1)
    print(data.head(2))

    return data


# 把ffm.pkl load进来，将上面的表转化为ffm格式
def transform_ffm_format(df, device_id):
    file_path = DIRECTORY_PATH + "ffm_{0}_{1}.pkl".format(DATA_START_DATE, DATA_END_DATE)
    with open(file_path, "rb") as f:
        ffm_format_pandas = pickle.load(f)
        data = ffm_format_pandas.transform(df)
        now = datetime.datetime.now().strftime("%Y-%m-%d-%H-%M")

        predict_file_name = DIRECTORY_PATH + "result/{0}_{1}DiaryTop2000.csv".format(device_id, now)
        data.to_csv(predict_file_name, index=False,header=None)
        return predict_file_name


# 将模型加载，预测，把预测日记的概率值按照降序排序，存到一个表里
def predict(user_profile):
    user_instance = feature_en(user_profile)
    user_instance_file_path = transform_ffm_format(user_instance, user_profile["device_id"])

    ffm_model = xl.create_ffm()
    ffm_model.setTest(user_instance_file_path)
    ffm_model.setSigmoid()
    ffm_model.predict(DIRECTORY_PATH + "model_{0}-{1}_lr{2}_lambda{3}.out".format(DATA_START_DATE,
                                                                                  DATA_END_DATE, lr, l2_lambda),
                      DIRECTORY_PATH + "result/{0}_output.txt".format(user_profile['device_id']))

    upload_predict(user_profile, user_instance)


def upload_predict(user_profile, instance):
    probabilities = pd.read_csv(DIRECTORY_PATH +
                                "result/{0}_output.txt".format(user_profile['device_id']), header=None)
    probabilities = probabilities.rename(columns={0: "prob"})
    probabilities["cid"] = instance['cid']
    probabilities = probabilities.sort_values(by="prob", ascending=False)

    wrapper_result(probabilities, user_profile['device_id'])


def wrapper_result(prob, device_id):
    prob = prob.head(500)
    prob.loc[:,"url"] = prob["cid"].apply(lambda x: "http://m.igengmei.com/diary_book/" + str(x[6:]) + '/')
    prob.to_csv(DIRECTORY_PATH + "result/{}_feed".format(device_id),index= False)


def router(device_id):
    user_profile, is_exist = fetch_user_profile(device_id)

    if is_exist:
        predict(user_profile)
    else:
        print('Sorry, we don\'t have you.')


if __name__ == "__main__":
    router(device_id='B42685C0-702F-4C54-8C9E-73AEC5F3AAEC')
